#!/usr/bin/python3


from os.path import dirname


def _add(files, file_data, name, name_only, prefix):
    if prefix is not None:
        name = prefix + name
    if name_only:
        files.append(name)
    else:
        files.append({'name': name,
                      'owner': file_data['owner'],
                      'group': file_data['group'],
                      'mode': file_data['mode'],
                      })


def fileslist(data, is_host=False, name_only=False, prefix=None):
    files = []
    if is_host:
        base_systemd = '/usr/local/lib'
    else:
        base_systemd = ''
        _add(files,
             {'owner': 'root', 'group': 'root', 'mode': '0755'},
             f'/tmpfiles.d/0rougail.conf',
             name_only,
             prefix,
             )
    for service, service_data in data.items():
        if not service_data['activate']:
            if service_data['manage']:
                if not service_data.get('undisable', False) and not service_data['engine'] and not service_data.get('target'):
                    _add(files,
                         {'owner': 'root', 'group': 'root', 'mode': '0755'},
                         base_systemd + '/systemd/system/' + service_data['doc'],
                         name_only,
                         prefix,
                         )
        else:
            if service_data['manage'] and service_data['engine']:
                _add(files,
                     {'owner': 'root', 'group': 'root', 'mode': '0755'},
                     base_systemd + '/systemd/system/' + service_data['doc'],
                     name_only,
                     prefix,
                     )
            if service_data.get('target'):
                _add(files,
                     {'owner': 'root', 'group': 'root', 'mode': '0755'},
                     f'/systemd/system/{service_data["target"]}.target.wants/{service_data["doc"]}',
                     name_only,
                     prefix,
                     )
            if 'overrides' in service_data:
                for override_data in service_data['overrides'].values():
                    _add(files,
                         {'owner': 'root', 'group': 'root', 'mode': '0755'},
                         base_systemd + '/systemd/system/' + override_data['name'] + '.d/rougail.conf',
                         name_only,
                         prefix,
                         )
            if 'ip' in service_data:
                _add(files,
                     {'owner': 'root', 'group': 'root', 'mode': '0755'},
                     base_systemd + '/systemd/system/' + service_data['doc'] + '.d/rougail_ip.conf',
                     name_only,
                     prefix,
                     )
        if 'files' not in service_data:
            continue
        for file_data in service_data['files'].values():
            if not file_data['activate'] or file_data['included'] == 'content':
                continue
            if isinstance(file_data['name'], list):
                for name in file_data['name']:
                    _add(files, file_data, name, name_only, prefix)
            else:
                _add(files, file_data, file_data['name'], name_only, prefix)

    return files


def directorieslist(data):
    directories = {'/usr/local/lib/systemd/system/'}
    for service, service_data in data.items():
        if 'files' not in service_data:
            continue
        for file_data in service_data['files'].values():
            if not file_data['activate']:
                continue
            if isinstance(file_data['name'], list):
                for name in file_data['name']:
                    directories.add(dirname(name))
            else:
                directories.add(dirname(file_data['name']))

    return list(directories)


def machineslist(data, only=None, only_name=False):
    srv = []
    if only is not None:
        if only not in data:
            raise Exception(f"cannot find {only} but only {data.keys()}")
        if only_name:
            srv.append(only)
        else:
            srv.append({'name': only,
                        'srv': data[only]['machine']['add_srv'],
                        }
                       )
    else:
        for host, host_data in data.items():
            if '.' not in host or not isinstance(host_data, dict) or 'general' not in host_data or host_data['general']['module_name'] == 'host':
                continue
            if only_name:
                srv.append(host)
            else:
                srv.append({'name': host,
                            'srv': host_data['machine']['add_srv'],
                            }
                           )
    return srv


def modulename(data, servername):
    return data[servername]['general']['module_name']


class FilterModule:
    def filters(self):
        return {
            'fileslist': fileslist,
            'directorieslist': directorieslist,
            'machineslist': machineslist,
            'modulename': modulename,
        }
